/**************************************************************
 * RobotModule.h
 *
 * Copyright (C) 2002 - Pacific Star Technologies Ltd 
 * All Rights Reserved. Unauthorized dissemination of this file
 * is strictly prohibited.
 *
 * Created: Aug 2, 2002
 *************************************************************/
#ifndef ROBOTMODULE_H_
#define ROBOTMODULE_H_

#define MAX_MODNAME_LEN      256 
#ifdef TRIAL_VERSION
#define MAX_PILL_TIMERS      1
#else
#define MAX_PILL_TIMERS      5
#endif

struct IRbtModule;
struct IRobot;

typedef struct {
  TCHAR name[MAX_MODNAME_LEN];
  union {
    CLSID clsid;
    struct IRbtModule *module;
  };
} MODULEDATA;

typedef struct {
  SYSTEMTIME nextExpire;
  ULARGE_INTEGER delta;
  DWORD repeats;
  DWORD left;
  DWORD refCnt;
  DWORD id;
} TIMERDATA;

typedef struct {
  DWORD timer;
  DWORD id;
  DWORD warn;
  LPTSTR desc;
} APPOINTMENTDATA;

typedef struct  {
  DWORD id;
  LPTSTR name;
  LPTSTR desc;
  BYTE numTimers;
  DWORD timers[MAX_PILL_TIMERS];
} PILLDATA;

DECLARE_INTERFACE_(IRbtModule, IUnknown)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;
  
  STDMETHOD(Init) (THIS_ IRobot *) PURE;
  STDMETHOD(Unload) (THIS) PURE;

  STDMETHOD (GetCLSID) (THIS_ CLSID *) PURE;
  STDMETHOD (GetName) (THIS_ LPTSTR, UINT) PURE;
};

DECLARE_INTERFACE_(IEnumRbtModules, IUnknown)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD (Next) (THIS_ ULONG, MODULEDATA *, ULONG *) PURE;
  STDMETHOD (Reset) (THIS) PURE;
  STDMETHOD (Skip) (THIS_ ULONG) PURE;
  STDMETHOD (Clone) (THIS_ IEnumRbtModules **) PURE;
};

////////////////////////////////////////////////////////////////
struct IRbtControl;

DECLARE_INTERFACE_(IRbtCmdGroup, IUnknown)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(Clear) (THIS) PURE;
  STDMETHOD(Move) (THIS_ RCMoveDir, RCMoveSpeed) PURE;
  STDMETHOD(MoveArm) (THIS_ RCSide, RCDirection) PURE;
  STDMETHOD(MoveHand) (THIS_ RCSide, BOOL) PURE;
  STDMETHOD(MoveHead) (THIS_ RCSide, BOOL) PURE;
  STDMETHOD(Tilt) (THIS_ RCDirection) PURE;
  STDMETHOD(ArmLock) (THIS_ BOOL) PURE;
  STDMETHOD(Light) (THIS) PURE;
  STDMETHOD(SetOpMode) (THIS_ RCOpMode) PURE;
  STDMETHOD(VideoOn) (THIS_ BOOL) PURE;
  STDMETHOD(SetVideoChannel) (THIS_ RCChannel) PURE;
  STDMETHOD(SetCtrlChannel) (THIS_ RCChannel) PURE;
  STDMETHOD(SetVolume) (THIS_ RCVolume) PURE;
  STDMETHOD(Commit) (THIS_ IRbtControl *) PURE;
};

DECLARE_INTERFACE_(IRbtControl, IRbtModule)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(Move) (THIS_ RCMoveDir, RCMoveSpeed) PURE;
  STDMETHOD(MoveArm) (THIS_ RCSide, RCDirection) PURE;
  STDMETHOD(MoveHand) (THIS_ RCSide, BOOL) PURE;
  STDMETHOD(MoveHead) (THIS_ RCSide, BOOL) PURE;
  STDMETHOD(Tilt) (THIS_ RCDirection) PURE;
  STDMETHOD(ArmLock) (THIS_ BOOL) PURE;
  STDMETHOD(Light) (THIS) PURE;
  STDMETHOD(SetOpMode) (THIS_ RCOpMode) PURE;
  STDMETHOD(VideoOn) (THIS_ BOOL) PURE;
  STDMETHOD(SetVideoChannel) (THIS_ RCChannel) PURE;
  STDMETHOD(SetCtrlChannel) (THIS_ RCChannel) PURE;
  STDMETHOD(SetVolume) (THIS_ RCVolume) PURE;
  STDMETHOD(Power) (THIS_ BOOL) PURE;

  STDMETHOD(CreateCmdGroup) (THIS_ IRbtCmdGroup **) PURE;
  
  STDMETHOD(GetStatus) (THIS_ RCStatus *) PURE;
  STDMETHOD(GetHandle) (THIS_ HROBOT *) PURE;
};

DECLARE_INTERFACE_(IRbtBaseAV, IRbtModule)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(SetVideoWindow) (THIS_ HWND) PURE;
  STDMETHOD(ProcessWindowMsg) (THIS_ UINT, WPARAM, LPARAM) PURE;

  STDMETHOD(StartVideo) (THIS) PURE;
  STDMETHOD(StopVideo) (THIS) PURE;
  STDMETHOD(StartCapture) (THIS) PURE;
  STDMETHOD(StopCapture) (THIS_ BOOL) PURE;
  STDMETHOD(Mute) (THIS) PURE;
  STDMETHOD(Unmute) (THIS) PURE;
  STDMETHOD(FullScreen) (THIS_ BOOL) PURE;

  // these are used to set properties
  STDMETHOD(SetVideoCompressor) (THIS_ IMoniker *) PURE;
  STDMETHOD(GetVideoCompressor) (THIS_ IMoniker **) PURE;
  STDMETHOD(SetAudioCompressor) (THIS_ IMoniker *) PURE;
  STDMETHOD(GetAudioCompressor) (THIS_ IMoniker **) PURE;
  STDMETHOD(SetVideoDevice) (THIS_ IMoniker *) PURE;
  STDMETHOD(GetVideoDevice) (THIS_ IMoniker **) PURE;
  STDMETHOD(SetAudioDevice) (THIS_ IMoniker *) PURE;
  STDMETHOD(GetAudioDevice) (THIS_ IMoniker **) PURE;
  STDMETHOD(SetCaptureFile) (THIS_ LPTSTR) PURE;
  STDMETHOD(GetCaptureFile) (THIS_ LPTSTR) PURE;
  STDMETHOD(SetTempFile) (THIS_ LPTSTR, DWORD) PURE;
  STDMETHOD(GetTempFile) (THIS_ LPTSTR, DWORD *) PURE;
  STDMETHOD(SetTimeLimit) (THIS_ DWORD) PURE;
  STDMETHOD(GetTimeLimit) (THIS_ DWORD *) PURE;
  STDMETHOD(SetFrameRate) (THIS_ DWORD) PURE;
  STDMETHOD(GetFrameRate) (THIS_ DWORD *) PURE;  

  STDMETHOD(LockFilterGraph) (THIS_ IUnknown **, BOOL) PURE;
  STDMETHOD(UnlockFilterGraph) (THIS_ BOOL) PURE;
}; 

DECLARE_INTERFACE_(IRbtTimeLapse, IRbtModule)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(SetCycleTime) (THIS_ DWORD) PURE;
  STDMETHOD(GetCycleTime) (THIS_ DWORD *) PURE;
  STDMETHOD(Start) (THIS) PURE;
  STDMETHOD(Stop) (THIS) PURE;
  STDMETHOD(SetSaveToFolder) (THIS_ LPCTSTR) PURE;
  STDMETHOD(GetSaveToFolder) (THIS_ LPTSTR) PURE;
  STDMETHOD(SetFileTemplate) (THIS_ LPCTSTR) PURE;
  STDMETHOD(GetFileTemplate) (THIS_ LPTSTR) PURE;
  STDMETHOD(SetCompressor) (THIS_ REFCLSID) PURE;
  STDMETHOD(GetCompressor) (THIS_ CLSID *) PURE;
  STDMETHOD(SetCompressImage) (THIS_ BOOL) PURE;
  STDMETHOD(GetCompressImage) (THIS) PURE;
  STDMETHOD(SetTimeStamp) (THIS_ BOOL) PURE;
  STDMETHOD(GetTimeStamp) (THIS) PURE;
};

DECLARE_INTERFACE_(IRbtEmailReader, IRbtModule)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(Enable) (THIS) PURE;
  STDMETHOD(Disable) (THIS) PURE;
  STDMETHOD(Read) (THIS) PURE;

  STDMETHOD(SetProfile) (THIS_ LPCTSTR) PURE;
  STDMETHOD(GetProfile) (THIS_ LPTSTR, UINT) PURE;
  STDMETHOD(SetPassword) (THIS_ LPCTSTR) PURE;
  STDMETHOD(GetPassword) (THIS_ LPTSTR, UINT) PURE;
  STDMETHOD(SetMarkAsRead) (THIS_ BOOL) PURE;
  STDMETHOD(GetMarkAsRead) (THIS_ BOOL *) PURE;
  STDMETHOD(SetNewOnly) (THIS_ BOOL) PURE;
  STDMETHOD(GetNewOnly) (THIS_ BOOL *) PURE;
  STDMETHOD(SetTriggerEvent) (THIS_ DWORD) PURE;
  STDMETHOD(GetTriggerEvent) (THIS_ DWORD *) PURE;
  STDMETHOD(SetTriggerParam) (THIS_ BOOL, DWORD) PURE;
  STDMETHOD(GetTriggerParam) (THIS_ DWORD *) PURE;
  STDMETHOD(SetStartTime) (THIS_ LPSYSTEMTIME) PURE;
  STDMETHOD(GetStartTime) (THIS_ LPSYSTEMTIME) PURE;
  STDMETHOD(SetEndTime) (THIS_ LPSYSTEMTIME) PURE;
  STDMETHOD(GetEndTime) (THIS_ LPSYSTEMTIME) PURE;
};

DECLARE_INTERFACE_(IRbtTTS, IRbtModule)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(Say) (THIS_ LPWSTR) PURE;
  STDMETHOD(Stop) (THIS) PURE;
  STDMETHOD(SetVoice) (THIS_ IUnknown *) PURE;
  STDMETHOD(GetVoice) (THIS_ IUnknown **) PURE;
  STDMETHOD(SetRate) (THIS_ DWORD) PURE;
  STDMETHOD(GetRate) (THIS_ DWORD *) PURE;
  STDMETHOD(SetVolume) (THIS_ DWORD) PURE;
  STDMETHOD(GetVolume) (THIS_ DWORD *) PURE;
};

DECLARE_INTERFACE_(IEnumRbtTimers, IUnknown)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(Reset) (THIS) PURE;
  STDMETHOD(Skip) (THIS_ ULONG) PURE;
  STDMETHOD(Next) (THIS_ ULONG, TIMERDATA *, ULONG *) PURE;
  STDMETHOD(Clone) (THIS_ IEnumRbtTimers **);
};

DECLARE_INTERFACE_(IRbtTimer, IRbtModule)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(Enable) (THIS_ BOOL) PURE;
  STDMETHOD(IsEnabled) (THIS) PURE;
  STDMETHOD(GetNumTimers) (THIS_ DWORD *) PURE;
  STDMETHOD(EnumTimers) (THIS_ IEnumRbtTimers **) PURE;
  STDMETHOD(AddTimer) (THIS_ TIMERDATA *) PURE;
  STDMETHOD(RefTimer) (THIS_ DWORD) PURE;
  STDMETHOD(UpdateTimer) (THIS_ TIMERDATA *) PURE;
  STDMETHOD(RemoveTimer) (THIS_ DWORD) PURE;
  STDMETHOD(GetTimer) (THIS_ TIMERDATA *) PURE;
};

DECLARE_INTERFACE_(IEnumRbtAppts, IUnknown) 
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(Reset) (THIS) PURE;
  STDMETHOD(Skip) (THIS_ ULONG) PURE;
  STDMETHOD(Next) (THIS_ ULONG, APPOINTMENTDATA *, ULONG *) PURE;
  STDMETHOD(Clone) (THIS_ IEnumRbtAppts **);
};

DECLARE_INTERFACE_(IRbtAppt, IRbtModule)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(GetNumAppts) (THIS_ DWORD *) PURE;
  STDMETHOD(EnumAppts) (THIS_ IEnumRbtAppts **) PURE;
  STDMETHOD(AddAppointment) (THIS_ APPOINTMENTDATA *) PURE;
  STDMETHOD(UpdateAppointment) (THIS_ APPOINTMENTDATA *) PURE;
  STDMETHOD(RemoveAppointment) (THIS_ DWORD) PURE;
  STDMETHOD(GetAppointment) (THIS_ APPOINTMENTDATA *) PURE;

  STDMETHOD(SetRemoveOld) (THIS_ BOOL) PURE;
  STDMETHOD(GetRemoveOld) (THIS) PURE;
  STDMETHOD(SetApptsFile) (THIS_ LPCTSTR) PURE;
  STDMETHOD(GetApptsFile) (THIS_ LPTSTR) PURE;
};

DECLARE_INTERFACE_(IEnumRbtPills, IUnknown) 
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(Reset) (THIS) PURE;
  STDMETHOD(Skip) (THIS_ ULONG) PURE;
  STDMETHOD(Next) (THIS_ ULONG, PILLDATA *, ULONG *) PURE;
  STDMETHOD(Clone) (THIS_ IEnumRbtPills **);
};


DECLARE_INTERFACE_(IRbtMedicine, IRbtModule)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(GetNumPills) (THIS_ DWORD *) PURE;
  STDMETHOD(EnumPills) (THIS_ IEnumRbtPills **) PURE;
  STDMETHOD(AddPill) (THIS_ PILLDATA *) PURE;
  STDMETHOD(UpdatePill) (THIS_ PILLDATA *) PURE;
  STDMETHOD(RemovePill) (THIS_ DWORD) PURE;
  STDMETHOD(GetPill) (THIS_ PILLDATA *) PURE;

  STDMETHOD(SetRepeats) (THIS_ UINT) PURE;
  STDMETHOD(GetRepeats) (THIS_ UINT *) PURE;
  STDMETHOD(SetPrompt) (THIS_ LPCTSTR) PURE;
  STDMETHOD(GetPrompt) (THIS_ LPTSTR, UINT) PURE;
  STDMETHOD(SetPillFile) (THIS_ LPCTSTR) PURE;
  STDMETHOD(GetPillFile) (THIS_ LPTSTR) PURE;
};

DECLARE_INTERFACE_(IRbtMotionDetect, IRbtModule)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(Enable) (THIS_ BOOL) PURE;
  STDMETHOD(IsEnabled) (THIS) PURE;
  STDMETHOD(SetThreshold) (THIS_ UINT) PURE;
  STDMETHOD(GetThreshold) (THIS_ UINT *) PURE;
  STDMETHOD(SetBlobCount) (THIS_ UINT) PURE;
  STDMETHOD(GetBlobCount) (THIS_ UINT *) PURE;
  STDMETHOD(SetSkip) (THIS_ UINT) PURE;
  STDMETHOD(GetSkip) (THIS_ UINT *) PURE;
  STDMETHOD(SetWaitFrames) (THIS_ UINT) PURE;
  STDMETHOD(GetWaitFrames) (THIS_ UINT *) PURE;
  STDMETHOD(SetSaveOnTrigger) (THIS_ BOOL) PURE;
  STDMETHOD(GetSaveOnTrigger) (THIS) PURE;
  STDMETHOD(SetSaveFolder) (THIS_ LPCTSTR) PURE;
  STDMETHOD(GetSaveFolder) (THIS_ LPTSTR) PURE;
  STDMETHOD(SetSaveFrames) (THIS_ UINT) PURE;
  STDMETHOD(GetSaveFrames) (THIS_ UINT *) PURE;
  STDMETHOD(SetTriggerFrames) (THIS_ UINT) PURE;
  STDMETHOD(GetTriggerFrames) (THIS_ UINT *) PURE;
  STDMETHOD(SetSpaceLimit) (THIS_ UINT) PURE;
  STDMETHOD(GetSpaceLimit) (THIS_ UINT *) PURE;
};

DECLARE_INTERFACE_(IRbtSoundDetect, IRbtModule)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(Enable) (THIS_ BOOL) PURE;
  STDMETHOD(IsEnabled) (THIS) PURE;
  STDMETHOD(SetThreshold) (THIS_ UINT) PURE;
  STDMETHOD(GetThreshold) (THIS_ UINT *) PURE;
  STDMETHOD(SetDuration) (THIS_ UINT) PURE;
  STDMETHOD(GetDuration) (THIS_ UINT *) PURE;
  STDMETHOD(SetSkip) (THIS_ UINT) PURE;
  STDMETHOD(GetSkip) (THIS_ UINT *) PURE;
  STDMETHOD(SetIgnore) (THIS_ UINT) PURE;
  STDMETHOD(GetIgnore) (THIS_ UINT *) PURE;
};

DECLARE_INTERFACE_(IRbtStoryTeller, IRbtModule)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(ReadFromFile) (THIS_ LPCTSTR) PURE;
  STDMETHOD(ReadFromClipboard) (THIS) PURE;
  STDMETHOD(Stop) (THIS) PURE;

  STDMETHOD(SetTimeout) (DWORD) PURE;
  STDMETHOD(GetTimeout) (DWORD *) PURE;
  STDMETHOD(SetProlog) (LPCTSTR) PURE;
  STDMETHOD(GetProlog) (LPTSTR *, UINT) PURE;
  STDMETHOD(SetEpilog) (LPCTSTR) PURE;
  STDMETHOD(GetEpilog) (LPTSTR *, UINT) PURE;
};
 
#endif
