/**************************************************************
 * DllMain.cpp 
 *
 * Copyright (C) 2002 - Pacific Star Technologies Ltd
 * All Rights Reserved. Unauthorized dissemination of this file
 * is strictly prohibited.
 *
 * Created: Nov 04, 2002
 *************************************************************/
#include <windows.h>
#include <ole2.h>
#include <olectl.h>
#include <tchar.h>
#include <initguid.h>

#define _DLLEXPORT_
#include "DllMain.h"
#include "RobotLib.h"
#include "Module.h"
#include "ObjFactory.h"
#include "Helper.h" 
#include "PropPage.h"
#include "StrCast.h"
#include "RobotGuid.h"


// TODO: Change name of CLSID
static const CLSID *moduleClsid = &CLSID_ModuleName;

// TODO: Change entries to correspond to module & prop page objects
static struct {
  TCHAR *verProgId;
  TCHAR *progId;
  TCHAR *desc;
  const CLSID *clsid;
} objects[] = {
  {_T("Robot.ModuleName.1"), _T("Robot.ModuleName"), _T("Module Description"), moduleClsid},
  {_T("Robot.PropPage1.1"), _T("Robot.PropPage1"), _T("Property Page Description"), &CLSID_PropPage1},
};

static UINT numObjs = sizeof(objects) / sizeof(objects[0]);
static const UINT GUID_SIZE = 128;

BOOL 
CheckUnicode()
{
  BOOL ok = TRUE;
  TCHAR userName[1024];
  DWORD size = 1024;

  if (!GetUserName(userName, &size))
    ok = (ERROR_CALL_NOT_IMPLEMENTED == GetLastError()) ? FALSE : TRUE;

  return ok;
}


BOOL WINAPI
DllMain(HINSTANCE inst, DWORD reason, LPVOID reserved)
{
  BOOL rv = TRUE;

  switch (reason) {
    case DLL_PROCESS_ATTACH:
      rv = FALSE;
      if (CheckUnicode()) {
        new CSvrState(inst);
        if (CSvrState::state) {
          rv = TRUE;
        }
      }
      break;
    case DLL_PROCESS_DETACH:
      if (CSvrState::state)
        delete CSvrState::state;
      break;
    case DLL_THREAD_ATTACH:
      break;
    case DLL_THREAD_DETACH:
      break;
    default:
      break;
  }
  return rv;
}

STDAPI
DllRegisterServer()
{
  TCHAR modPath[MAX_PATH];
  HRESULT hr;
 
  GetModuleFileName(CSvrState::state->dllInst, modPath, 
      sizeof(modPath)/sizeof(TCHAR));

  for (UINT i = 0; i < numObjs; i++)  {
    hr = RbtRegisterObject(modPath, objects[i].verProgId, objects[i].progId, objects[i].desc, *(objects[i].clsid));
    if (FAILED(hr)) return hr;
  }

  hr = RbtRegisterModule(moduleName, *moduleClsid);

  return hr; 
}

STDAPI
DllUnregisterServer()
{
  HRESULT hr;
  for (UINT i = 0; i < numObjs; i++) {
    hr = RbtUnregisterObject(objects[i].verProgId, objects[i].progId, *(objects[i].clsid));
    if (FAILED(hr)) return hr;
  }

  hr = RbtUnregisterModule(moduleName);
  return hr;
}

STDAPI
DllGetClassObject(REFCLSID clsid, REFIID iid, void **ppv)
{
  HRESULT hr = CLASS_E_CLASSNOTAVAILABLE;
  IUnknown *pCob = NULL;

  if (*moduleClsid == clsid) {
    hr = E_OUTOFMEMORY;
    pCob = new CModFactory(NULL);
  } else if (CLSID_PropPage1 == clsid) {
    hr = E_OUTOFMEMORY;
    pCob = new CPropFactory(NULL, clsid);
  } 
  
  if (NULL != pCob) {
    hr = pCob->QueryInterface(iid, ppv);
    if (FAILED(hr)) {
      delete pCob;
    } else {
      CSvrState::state->IncObjs();
    }
  }
  return hr;
}

STDAPI
DllCanUnloadNow()
{
  if (0L == CSvrState::state->objCnt && 0L == CSvrState::state->lockCnt) {
    return S_OK;
  } else {
    return S_FALSE;
  }
}
